//+------------------------------------------------------------------+
//|                      OM NAMAH SIVAY                              |
//|        OM NAMAH SIVAY              OM NAMAH SIVAY                |
//|    OM NAMAH SIVAY    OM NAMAH SIVAY    OM NAMAH SIVAY            |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, OM Corp."
#property link "http://www.jatinpruthvi.co.cc"
//+------------------------------------------------------------------+
//| CLASS DECLARATION                                                |
//+------------------------------------------------------------------+
class MySymbolExpert {
    //--- private members
private:
    //   double LOTS; //Lots or volume to Trade
    //  double TradePct; //Percentage of Account Free Margin to trade
    double ADX_min; // ADX Minimum value
    int ADX_handle; // ADX Handle
    int MA_handle; // Moving Average Handle
    int  BolBandsHandle , demaHandle , RVI_handle ;
    double plus_DI[]; // array to hold ADX +DI values for each bars
    double minus_DI[]; // array to hold ADX -DI values for each bars
    double MA_val[]; // array to hold Moving Average values for each bars
    double ADX_val[]; // array to hold ADX values for each bars
    double Closeprice ,  OpenPrice; // variable to hold the previous bar closed price
    double mypricearray0[]; //rvi
    double mypricearray1[]; // rvi
    double BBUp[],BBLow[],BBMidle[],demaVal[];
    string symbol; // variable to hold the current symbol name
    ENUM_TIMEFRAMES period; // variable to hold the current timeframe value
    string Errormsg; // variable to hold our error messages
    int Errcode; // variable to hold our error codes
    bool useRVI;
    bool useADX;
    bool useMA , useBB,useDEMA;
    //--- Public member/functions
public:
    void MySymbolExpert(); // Class Constructor
    void setSymbol(string syb)
    {
        symbol = syb;
    } // function to set current symbol
    void setPeriod(ENUM_TIMEFRAMES prd)
    {
        period = prd;
    } // function to set current symbol timeframe/period

    void setadxmin(double adx)
    {
        ADX_min = adx;
    } // function to set ADX Minimum values
    void doInit(int adx_period, int ma_period, int nRVI_PERIOD,int BB_Band,int DEMA_PERIOD);
    void doUninit(); // function to be used at EA de-initialization
    bool checkBuy(); // function to check for Buy conditions
    bool checkSell(); // function to check for Sell conditions
    ENUM_ORDER_TYPE getBuySellSignal();
    bool doMQLRates();
    bool buyClose();
    bool sellClose();
    //--- Protected members
protected:
    void showError(string msg, int ercode); // function for use to display error messages
    void getBuffers(); // function for getting Indicator buffers

}; // end of class declaration
//+------------------------------------------------------------------+
//| Definition of our Class/member functions                         |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|  This CLASS CONSTRUCTOR                                          |
//|  *Does not have any input parameters                             |
//|  *Initilizes all the necessary variables                         |
//+------------------------------------------------------------------+
void MySymbolExpert::MySymbolExpert()
{
    //---initialize all necessary variables

    ZeroMemory(ADX_val);
    ZeroMemory(MA_val);
    ZeroMemory(plus_DI);
    ZeroMemory(minus_DI);
    ZeroMemory(mypricearray0);
    ZeroMemory(mypricearray1);
    ZeroMemory(BBUp);
    ZeroMemory(BBLow);
    ZeroMemory(BBMidle);
    ZeroMemory(demaVal);
    Errormsg = "";
    Errcode = 0;
}
//+-----------------------------------------------------------------------+
//| OUR PUBLIC FUNCTIONS                                                  |
//+-----------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| DOINIT FUNCTION                                                  |
//| *Takes the ADX indicator's Period and Moving Average indicator's |
//| period as input parameters                                       |
//| *To be used in the OnInit() function of our EA                   |
//+------------------------------------------------------------------+
void MySymbolExpert::doInit(int adx_period, int ma_period, int nRVI_PERIOD,int BB_Band,int DEMA_PERIOD)
{
    useRVI = nRVI_PERIOD > 0;
    useADX = adx_period > 0;
    useMA = ma_period > 0;
    useBB = BB_Band > 0; 
    useDEMA = DEMA_PERIOD > 0;
    //--- get handle for ADX indicator
    if (useADX)
        ADX_handle = iADX(symbol, period, adx_period);
    //--- get the handle for Moving Average indicator
    if (useMA)
        MA_handle = iMA(symbol, period, ma_period, 0, MODE_EMA, PRICE_CLOSE);
    //--- what if handle returns Invalid Handle
    if (useRVI)
        RVI_handle = iRVI(symbol, period, nRVI_PERIOD);

    if(useBB){
          BolBandsHandle=iBands(symbol,period,BB_Band,0,2,PRICE_CLOSE);
    }
    if(useDEMA){
          demaHandle=iDEMA(symbol,period,DEMA_PERIOD,0,PRICE_CLOSE);    
    }
    if ((useADX && ADX_handle < 0) || (useMA && MA_handle < 0) || (useRVI && RVI_handle < 0)|| (useBB && BolBandsHandle < 0 ) || (useDEMA && demaHandle < 0)) {
        Errormsg = "Error Creating Handles for indicators";
        Errcode = GetLastError();
        showError(Errormsg, Errcode);
    }

    //--- set Arrays as series
    //--- the ADX values arrays
    if (useADX) {
        ArraySetAsSeries(ADX_val, true);
        //--- the +DI value arrays
        ArraySetAsSeries(plus_DI, true);
        //--- the -DI value arrays
        ArraySetAsSeries(minus_DI, true);
    }

    //--- the MA values arrays
    if (useMA)
        ArraySetAsSeries(MA_val, true);

    if (useRVI) {
        ArraySetAsSeries(mypricearray0, true);
        ArraySetAsSeries(mypricearray1, true);
    }
    if(useBB){
          ArraySetAsSeries(BBUp,true);
   ArraySetAsSeries(BBLow,true);
   ArraySetAsSeries(BBMidle,true);
    }
    if(useDEMA){
            ArraySetAsSeries(demaVal,true);
    }
    
}
//+------------------------------------------------------------------+
//|  SHOWERROR FUNCTION                                              |
//|  *Input Parameters - Error Message, Error Code                   |
//+------------------------------------------------------------------+
void MySymbolExpert::showError(string msg, int ercode)
{
    Print(msg, "-error:", ercode, "!!"); // display error
}

//+------------------------------------------------------------------+
//|  GETBUFFERS FUNCTION                                             |
//|  *No input parameters                                            |
//|  *Uses the class data members to get indicator's buffers         |
//+------------------------------------------------------------------+
void MySymbolExpert::getBuffers()
{
    if (useADX) {
        CopyBuffer(ADX_handle, 0, 0, 3, ADX_val);
        CopyBuffer(ADX_handle, 1, 0, 3, plus_DI);
        CopyBuffer(ADX_handle, 2, 0, 3, minus_DI);
    }
    if (useMA) {
        CopyBuffer(MA_handle, 0, 0, 3, MA_val);
    }
    if (useRVI) {
        CopyBuffer(RVI_handle, 0, 0, 3, mypricearray0);
        CopyBuffer(RVI_handle, 1, 0, 3, mypricearray1);
    }
    if(useBB){
CopyBuffer(BolBandsHandle,2,0,3,BBLow)         ;
CopyBuffer(BolBandsHandle,0,0,3,BBMidle);
CopyBuffer(BolBandsHandle,1,0,3,BBUp);
         
    }
    if(useDEMA){
            CopyBuffer(demaHandle,0,0,3,demaVal);
    }
}

//+------------------------------------------------------------------+
//|  DOUNINIT FUNCTION                                               |
//|  *No input parameters                                            |
//|  *Used to release ADX and MA indicators handles                  |
//+------------------------------------------------------------------+
void MySymbolExpert::doUninit()
{
    //--- release our indicator handles
    if (useADX)
        IndicatorRelease(ADX_handle);
    if (useMA)
        IndicatorRelease(MA_handle);
    if (useRVI) {
        IndicatorRelease(RVI_handle);
    }
    if(useBB){
        IndicatorRelease(BolBandsHandle);     
    }
    if(useDEMA){
        IndicatorRelease(demaHandle);
    }
}

bool MySymbolExpert::doMQLRates()
{
    MqlRates mrate[];
    ArraySetAsSeries(mrate, true);
    if (CopyRates(symbol, period, 0, 3, mrate) < 0) {
        Print("Error copying rates/history data - error:", GetLastError(), "!!");
        return false;
    }
    Closeprice = mrate[1].close;
    OpenPrice = mrate[1].open ;
    return true;
}
// 0 - nothing , 1-Buy , 2-Sell
ENUM_ORDER_TYPE MySymbolExpert::getBuySellSignal()
{
    ENUM_ORDER_TYPE nSignal = -1;
    int count = 0;
    if (checkBuy()) {
        nSignal = ORDER_TYPE_BUY;
        count++;
    }
    if (checkSell()) {
        nSignal = ORDER_TYPE_SELL;
        count++;
    }
    if (count != 0 && count < 2) {
        return nSignal;
    }
    else {
        return -1;
    }
}
//+------------------------------------------------------------------+
//| CHECKBUY FUNCTION                                                |
//| *No input parameters                                             |
//| *Uses the class data members to check for Buy setup              |
//| based on the defined trade strategy                              |
//| *Returns TRUE if Buy conditions are met or FALSE if not met      |
//+------------------------------------------------------------------+
bool MySymbolExpert::checkBuy()
{
    bool result = true;
    /*
    Check for a Long/Buy Setup : MA increasing upwards,
    previous price close above MA, ADX > ADX min, +DI > -DI
    */
    getBuffers();

    //--- declare bool type variables to hold our Buy Conditions
    // previous price closed above MA
if(useBB){
    if(buyClose()) return false;
         result = result && (Closeprice > BBLow[1] && OpenPrice < BBLow[1]);
    }
    if(useDEMA){
        result = result && ( demaVal[0]>demaVal[1] && demaVal[1]>demaVal[2]);
    }
    //--- Putting all together

    if (useRVI) {
        bool rviBuy = true;
        double rVal0 = NormalizeDouble(mypricearray0[0], 3);
        double rVal1 = NormalizeDouble(mypricearray1[0], 3);
        //buy
        if ((rVal0 > rVal1 && (rVal0 > 0)) || (rVal0 > rVal1 && (rVal1 < 0))) {
            rviBuy = true;
        }
        else {
            rviBuy = false;
        }
        result = result && rviBuy;
    }
    if (result == false) {
        return result;
    }
    if (useADX) {
        bool ADXBuy = false;
        bool Buy_Condition_3 = (ADX_val[0] > ADX_min); // Current ADX value greater than minimum ADX value
        bool Buy_Condition_4 = (plus_DI[0] > minus_DI[0]); // +DI greater than -DI
        if (Buy_Condition_3 && Buy_Condition_4) {
            ADXBuy = true;
        }
        else {
            ADXBuy = false;
        }
        result = result && ADXBuy;
    }
    if (result == false) {
        return result;
    }
    if (useMA) {
        bool maBuy = true;
        if (!doMQLRates()) {
            maBuy = false;
        }
        if (maBuy) {
            bool Buy_Condition_1 = (MA_val[0] > MA_val[1]) && (MA_val[1] > MA_val[2]); // MA Increasing upwards
            bool Buy_Condition_2 = (Closeprice > MA_val[1]);
            if (Buy_Condition_1 && Buy_Condition_2) {
                maBuy = true;
            }
            else {
                maBuy = false;
            }
        }
        result = result && maBuy;
    }
    
    return result;
}

//+------------------------------------------------------------------+
//| CHECKSELL FUNCTION                                               |
//| *No input parameters                                             |
//| *Uses the class data members to check for Sell setup             |
//|  based on the defined trade strategy                             |
//| *Returns TRUE if Sell conditions are met or FALSE if not met     |
//+------------------------------------------------------------------+
bool MySymbolExpert::checkSell()
{
    bool result = true;
    /*
    Check for a Short/Sell Setup : MA decreasing downwards,
    previous price close below MA, ADX > ADX min, -DI > +DI
    */
    getBuffers();
    //--- declare bool type variables to hold our Sell Conditions
 
     if(useBB){
         if(sellClose()) return false;
         result = result && (Closeprice < BBUp[1] && OpenPrice > BBUp[1]);
    }
    if(useDEMA){
        result = result && ( demaVal[0]<demaVal[1] && demaVal[1]<demaVal[2]);
    }
     
    if (useADX) {
        bool adxsell = false;
        bool Sell_Condition_3 = (ADX_val[0] > ADX_min); // Current ADX value greater than minimum ADX
        bool Sell_Condition_4 = (plus_DI[0] < minus_DI[0]); // -DI greater than +DI
        if (Sell_Condition_3 && Sell_Condition_4) {
            adxsell = true;
        }
        else {
            adxsell = false;
        }
        result = result && adxsell;
    }
    if (result == false) {
        return result;
    }
    if (useMA) {
        bool maResult = true;
        if (!doMQLRates()) {
            maResult = false;
        }
        if (maResult) {
            bool Sell_Condition_1 = (MA_val[0] < MA_val[1]) && (MA_val[1] < MA_val[2]); // MA decreasing downwards
            bool Sell_Condition_2 = (Closeprice < MA_val[1]); // Previous price closed below MA
            if (Sell_Condition_1 && Sell_Condition_2) {
                maResult = true;
            }
            else {
                maResult = false;
            }
        }
        result = result && maResult;
    }
    if (result == false) {
        return result;
    }
    if (useRVI) {
        bool rviSell = false;
        double rVal0 = NormalizeDouble(mypricearray0[0], 3);
        double rVal1 = NormalizeDouble(mypricearray1[0], 3);
        if ((rVal0 < rVal1 && (rVal1 < 0)) || (rVal0 < rVal1 && (rVal0 > 0))) {
            rviSell = true;
        }
        else {
            rviSell = false;
        }
        result = result && rviSell;
    }

    //--- putting all together buyClose sellClose

    return result;
}

bool MySymbolExpert::buyClose(){
    bool result = true;
    if(useBB){
        bool bbBuy = true;
         if (!doMQLRates()) {
            bbBuy = false;
        }
        if(bbBuy){
            bbBuy = Closeprice < BBUp[1] && OpenPrice > BBUp[1];
        }
         result = result && bbBuy;
    }
    return result;
}
bool MySymbolExpert::sellClose(){
    bool result = true;
    if(useBB){
        bool bbBuy = true;
         if (!doMQLRates()) {
            bbBuy = false;
        }
        if(bbBuy){
            bbBuy = Closeprice > BBLow[1] && OpenPrice < BBLow[1];
        }
         result = result && bbBuy;
    }
    return result;
}

//+------------------------------------------------------------------+
//| OPENBUY FUNCTION                                                 |
//| *Has Input parameters - order type, Current ASK price,           |
//|  Stop Loss, Take Profit, deviation, comment                      |
//| *Checks account free margin before pacing trade if trader chooses|
//| *Prints of a success if position is opened or shows error        |
//+------------------------------------------------------------------+